@@ -66,7 +66,9 @@ struct dep {
 	struct dep *next;
 	// bool?
 	int type;
+	// default value
 	int int_val;
+	int def_val;
 	// hitflag?
 	int hitflag;
 	int own_config;
@@ -294,10 +296,17 @@ static int read_file_1(const char *filename, struct cond **template)
 		/* ignore some keywords */
 		if (buffer[0] == '#')
 			continue;
-		if (!strncmp(buffer, "        ", 8)) {
-			buffer[0] = '\t';
-			for (idx = 8; idx <= strlen(buffer); idx++)
-				buffer[idx-7] = buffer[idx];
+		if (buffer[0] != '\t') {
+			for (c = 0; c < 8; c++) {
+				if (buffer[c] == '\t') {
+					buffer[0] = '\t';
+					memmove(buffer + 1, buffer + c + 1,
+						strlen(buffer) - c);
+					break;
+				}
+				if (buffer[c] != ' ')
+					break;
+			}
 		}
 		if (!strncmp(buffer, "endmenu", 7) ||
 		    !strncmp(buffer, "endif", 5)) {
@@ -368,6 +377,10 @@ static int read_file_1(const char *filename, struct cond **template)
 				dep->type = TYPE_BOOL;
 				if (buffer[5])
 					dep->selectable = 1;
+			} else if (!strncmp(buffer, "\tdef_bool ", 10)) {
+				dep->type = TYPE_BOOL;
+				if (buffer[10] == 'y')
+					dep->def_val = 1;
 			} else if (!strncmp(buffer, "\tint ", 5)) {
 				dep->type = TYPE_INT;
 				/*dep->selectable = 1;*/ /* exception */
@@ -383,6 +396,22 @@ static int read_file_1(const char *filename, struct cond **template)
 					for (; *p && !isdigit(*p); p++)
 						;
 					dep->int_val = strtol(p, NULL, 0);
+				} else {
+					switch (buffer[9]) {
+					case 'y':
+					case 'm':
+						dep->def_val = 1;
+						break;
+					case 'n':
+						dep->def_val = 0;
+						break;
+					default:
+						fprintf(stderr, "can't handle 'default %s', translating as depends on\n",
+							buffer + 9);
+						dep->def_val = 1;
+						add_dep(dep, buffer + 9);
+						break;
+					}
 				}
 			}
 			continue;
@@ -1172,8 +1201,10 @@ static void process_dep_acinclude(struct dep *tempdep, int slave,
 	struct sel *sel;
 
 	if (!tempdep->selectable) {
-		if (tempdep->type != TYPE_INT && !slave)
-			return;
+		if (tempdep->type != TYPE_INT && !slave) {
+			if (!tempdep->cond || !tempdep->def_val)
+				return;
+		}
 	}
 	if (!is_toplevel(tempdep) || tempdep->processed)
 		return;
@@ -1225,16 +1256,18 @@ static void process_dep_acinclude(struct dep *tempdep, int slave,
 		free(text);
 	} else if (tempdep->type == TYPE_INT) {
 		put_define = 1;
-	} else {
-		if (!tempdep->sel)
-			return;
+	} else if (tempdep->sel) {
 		text = convert_to_config_uppercase("CONFIG_", tempdep->name);
 		if (!text)
 			return;
 		printf("  if test \"$CONFIG_%s\" = \"m\" -o \"$CONFIG_%s\" = \"y\"; then\n", text, text);
 		free(text);
 		put_topif = 1;
-	}
+	} else if (tempdep->cond && tempdep->def_val) {
+		/* "def_bool yes" and depends on ... */
+		put_define = 1;
+	} else
+		return;
 	put_if = 0;
 	for (cond = tempdep->cond, cond_prev = NULL; cond;
 	     cond_prev = cond, cond = cond->next) {
